@@ -786,8 +786,8 @@ update_scale (gpointer data)
 
   if (stats) {
     if (clock) {
-      g_print ("clock:                  %13llu  (%s)\n", position,
-          gst_object_get_name (GST_OBJECT (clock)));
+      g_print ("clock:                  %13" G_GUINT64_FORMAT "  (%s)\n",
+          position, gst_object_get_name (GST_OBJECT (clock)));
     }
 
     if (elem_seek) {
