@@ -27,6 +27,8 @@
 #include "_stdint.h"
 
 #include <gst/gst.h>
+#include <gst/audio/multichannel.h>
+
 #include <a52dec/a52.h>
 #include <a52dec/mm_accel.h>
 #include "gsta52dec.h"
@@ -39,6 +41,12 @@ static GstElementDetails gst_a52dec_details = {
   "David I. Lehn <dlehn@users.sourceforge.net>",
 };
 
+#ifdef LIBA52_DOUBLE
+#define SAMPLE_WIDTH 64
+#else
+#define SAMPLE_WIDTH 32
+#endif
+
 GST_DEBUG_CATEGORY_STATIC (a52dec_debug);
 #define GST_CAT_DEFAULT (a52dec_debug)
 
@@ -55,10 +63,6 @@ enum
   ARG_DRC
 };
 
-/*
- * "audio/a52", "audio/x-a52" and "audio/ac3" should not be used (deprecated names)
- * Only use "audio/x-ac3" in new code.
- */
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -68,19 +72,18 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/x-raw-int, "
+    GST_STATIC_CAPS ("audio/x-raw-float, "
         "endianness = (int) BYTE_ORDER, "
-        "signed = (boolean) true, "
-        "width = (int) 16, "
-        "depth = (int) 16, "
-        "rate = (int) [ 4000, 48000 ], " "channels = (int) [ 1, 6 ]")
+        "width = (int) " G_STRINGIFY (SAMPLE_WIDTH) ", "
+        "rate = (int) [ 4000, 96000 ], "
+        "channels = (int) [ 1, 6 ], " "buffer-frames = (int) 0")
     );
 
 static void gst_a52dec_base_init (gpointer g_class);
 static void gst_a52dec_class_init (GstA52DecClass * klass);
 static void gst_a52dec_init (GstA52Dec * a52dec);
 
-static void gst_a52dec_loop (GstElement * element);
+static void gst_a52dec_chain (GstPad * pad, GstData * data);
 static GstElementStateReturn gst_a52dec_change_state (GstElement * element);
 
 static void gst_a52dec_set_property (GObject * object, guint prop_id,
@@ -153,12 +156,14 @@ gst_a52dec_class_init (GstA52DecClass * klass)
 static void
 gst_a52dec_init (GstA52Dec * a52dec)
 {
+  GST_FLAG_SET (GST_ELEMENT (a52dec), GST_ELEMENT_EVENT_AWARE);
+
   /* create the sink and src pads */
   a52dec->sinkpad =
       gst_pad_new_from_template (gst_element_get_pad_template (GST_ELEMENT
           (a52dec), "sink"), "sink");
+  gst_pad_set_chain_function (a52dec->sinkpad, gst_a52dec_chain);
   gst_element_add_pad (GST_ELEMENT (a52dec), a52dec->sinkpad);
-  gst_element_set_loop_function ((GstElement *) a52dec, gst_a52dec_loop);
 
   a52dec->srcpad =
       gst_pad_new_from_template (gst_element_get_pad_template (GST_ELEMENT
@@ -166,225 +171,175 @@ gst_a52dec_init (GstA52Dec * a52dec)
   gst_pad_use_explicit_caps (a52dec->srcpad);
   gst_element_add_pad (GST_ELEMENT (a52dec), a52dec->srcpad);
 
-  GST_FLAG_SET (GST_ELEMENT (a52dec), GST_ELEMENT_EVENT_AWARE);
   a52dec->dynamic_range_compression = FALSE;
-}
-
-/* BEGIN modified a52dec conversion code */
-
-static inline int16_t
-convert (int32_t i)
-{
-  if (i > 0x43c07fff)
-    return 32767;
-  else if (i < 0x43bf8000)
-    return -32768;
-  else
-    return i - 0x43c00000;
-}
-
-static inline void
-float_to_int (float *_f, int16_t * s16, int flags)
-{
-  int i;
-  int32_t *f = (int32_t *) _f;
-
-  switch (flags) {
-    case A52_MONO:
-      for (i = 0; i < 256; i++) {
-        s16[5 * i] = s16[5 * i + 1] = s16[5 * i + 2] = s16[5 * i + 3] = 0;
-        s16[5 * i + 4] = convert (f[i]);
-      }
-      break;
-    case A52_CHANNEL:
-    case A52_STEREO:
-    case A52_DOLBY:
-      for (i = 0; i < 256; i++) {
-        s16[2 * i] = convert (f[i]);
-        s16[2 * i + 1] = convert (f[i + 256]);
-      }
-      break;
-    case A52_3F:
-      for (i = 0; i < 256; i++) {
-        s16[5 * i] = convert (f[i]);
-        s16[5 * i + 1] = convert (f[i + 512]);
-        s16[5 * i + 2] = s16[5 * i + 3] = 0;
-        s16[5 * i + 4] = convert (f[i + 256]);
-      }
-      break;
-    case A52_2F2R:
-      for (i = 0; i < 256; i++) {
-        s16[4 * i] = convert (f[i]);
-        s16[4 * i + 1] = convert (f[i + 256]);
-        s16[4 * i + 2] = convert (f[i + 512]);
-        s16[4 * i + 3] = convert (f[i + 768]);
-      }
-      break;
-    case A52_3F2R:
-      for (i = 0; i < 256; i++) {
-        s16[5 * i] = convert (f[i]);
-        s16[5 * i + 1] = convert (f[i + 512]);
-        s16[5 * i + 2] = convert (f[i + 768]);
-        s16[5 * i + 3] = convert (f[i + 1024]);
-        s16[5 * i + 4] = convert (f[i + 256]);
-      }
-      break;
-    case A52_MONO | A52_LFE:
-      for (i = 0; i < 256; i++) {
-        s16[6 * i] = s16[6 * i + 1] = s16[6 * i + 2] = s16[6 * i + 3] = 0;
-        s16[6 * i + 4] = convert (f[i + 256]);
-        s16[6 * i + 5] = convert (f[i]);
-      }
-      break;
-    case A52_CHANNEL | A52_LFE:
-    case A52_STEREO | A52_LFE:
-    case A52_DOLBY | A52_LFE:
-      for (i = 0; i < 256; i++) {
-        s16[6 * i] = convert (f[i + 256]);
-        s16[6 * i + 1] = convert (f[i + 512]);
-        s16[6 * i + 2] = s16[6 * i + 3] = s16[6 * i + 4] = 0;
-        s16[6 * i + 5] = convert (f[i]);
-      }
-      break;
-    case A52_3F | A52_LFE:
-      for (i = 0; i < 256; i++) {
-        s16[6 * i] = convert (f[i + 256]);
-        s16[6 * i + 1] = convert (f[i + 768]);
-        s16[6 * i + 2] = s16[6 * i + 3] = 0;
-        s16[6 * i + 4] = convert (f[i + 512]);
-        s16[6 * i + 5] = convert (f[i]);
-      }
-      break;
-    case A52_2F2R | A52_LFE:
-      for (i = 0; i < 256; i++) {
-        s16[6 * i] = convert (f[i + 256]);
-        s16[6 * i + 1] = convert (f[i + 512]);
-        s16[6 * i + 2] = convert (f[i + 768]);
-        s16[6 * i + 3] = convert (f[i + 1024]);
-        s16[6 * i + 4] = 0;
-        s16[6 * i + 5] = convert (f[i]);
-      }
-      break;
-    case A52_3F2R | A52_LFE:
-      for (i = 0; i < 256; i++) {
-        s16[6 * i] = convert (f[i + 256]);
-        s16[6 * i + 1] = convert (f[i + 768]);
-        s16[6 * i + 2] = convert (f[i + 1024]);
-        s16[6 * i + 3] = convert (f[i + 1280]);
-        s16[6 * i + 4] = convert (f[i + 512]);
-        s16[6 * i + 5] = convert (f[i]);
-      }
-      break;
-  }
+  a52dec->cache = NULL;
 }
 
 static int
-gst_a52dec_channels (int flags)
+gst_a52dec_channels (int flags, GstAudioChannelPosition ** _pos)
 {
   int chans = 0;
+  GstAudioChannelPosition *pos = NULL;
+
+  /* allocated just for safety. Number makes no sense */
+  if (_pos) {
+    pos = g_new (GstAudioChannelPosition, 6);
+    *_pos = pos;
+  }
 
   if (flags & A52_LFE) {
     chans += 1;
+    if (pos) {
+      pos[0] = GST_AUDIO_CHANNEL_POSITION_LFE;
+    }
   }
   flags &= A52_CHANNEL_MASK;
   switch (flags) {
     case A52_3F2R:
+      if (pos) {
+        pos[0 + chans] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
+        pos[1 + chans] = GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER;
+        pos[2 + chans] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
+        pos[3 + chans] = GST_AUDIO_CHANNEL_POSITION_REAR_LEFT;
+        pos[4 + chans] = GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT;
+      }
       chans += 5;
       break;
     case A52_2F2R:
+      if (pos) {
+        pos[0 + chans] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
+        pos[1 + chans] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
+        pos[2 + chans] = GST_AUDIO_CHANNEL_POSITION_REAR_LEFT;
+        pos[3 + chans] = GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT;
+      }
+      chans += 4;
+      break;
     case A52_3F1R:
+      if (pos) {
+        pos[0 + chans] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
+        pos[1 + chans] = GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER;
+        pos[2 + chans] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
+        pos[3 + chans] = GST_AUDIO_CHANNEL_POSITION_REAR_CENTER;
+      }
       chans += 4;
       break;
     case A52_2F1R:
-    case A52_3F:
+      if (pos) {
+        pos[0 + chans] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
+        pos[1 + chans] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
+        pos[2 + chans] = GST_AUDIO_CHANNEL_POSITION_REAR_CENTER;
+      }
       chans += 3;
-    case A52_CHANNEL:
+      break;
+    case A52_3F:
+      if (pos) {
+        pos[0 + chans] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
+        pos[1 + chans] = GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER;
+        pos[2 + chans] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
+      }
+      chans += 3;
+      break;
+      /*case A52_CHANNEL: */
     case A52_STEREO:
     case A52_DOLBY:
+      if (pos) {
+        pos[0 + chans] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
+        pos[1 + chans] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
+      }
       chans += 2;
       break;
     default:
       /* error */
       g_warning ("a52dec invalid flags %d", flags);
+      g_free (pos);
       return 0;
   }
+
   return chans;
 }
 
 static int
-gst_a52dec_push (GstPad * srcpad, int flags, sample_t * _samples,
+gst_a52dec_push (GstPad * srcpad, int flags, sample_t * samples,
     GstClockTime timestamp)
 {
   GstBuffer *buf;
-  int chans;
+  int chans, n, c;
 
-#ifdef LIBA52_DOUBLE
-  float samples[256 * 6];
-  int i;
-
-  for (i = 0; i < 256 * 6; i++)
-    samples[i] = _samples[i];
-#else
-  float *samples = _samples;
-#endif
-
-  flags &= A52_CHANNEL_MASK | A52_LFE;
-
-  chans = gst_a52dec_channels (flags);
+  flags &= (A52_CHANNEL_MASK | A52_LFE);
+  chans = gst_a52dec_channels (flags, NULL);
   if (!chans) {
     return 1;
   }
 
   buf = gst_buffer_new ();
-  GST_BUFFER_SIZE (buf) = sizeof (int16_t) * 256 * chans;
+  GST_BUFFER_SIZE (buf) = 256 * chans * (SAMPLE_WIDTH / 8);
   GST_BUFFER_DATA (buf) = g_malloc (GST_BUFFER_SIZE (buf));
+  for (n = 0; n < 256; n++) {
+    for (c = 0; c < chans; c++) {
+      ((sample_t *) GST_BUFFER_DATA (buf))[n * chans + c] =
+          samples[c * 256 + n];
+    }
+  }
   GST_BUFFER_TIMESTAMP (buf) = timestamp;
-  float_to_int (samples, (int16_t *) GST_BUFFER_DATA (buf), flags);
 
   gst_pad_push (srcpad, GST_DATA (buf));
 
   return 0;
 }
 
-/* END modified a52dec conversion code */
-
-static void
-gst_a52dec_reneg (GstPad * pad, int channels, int rate)
+static gboolean
+gst_a52dec_reneg (GstPad * pad)
 {
-  GST_INFO ("a52dec: reneg channels:%d rate:%d\n", channels, rate);
+  GstAudioChannelPosition *pos;
+  GstA52Dec *a52dec = GST_A52DEC (gst_pad_get_parent (pad));
+  gint channels = gst_a52dec_channels (a52dec->using_channels, &pos);
+  GstCaps *caps;
 
-  gst_pad_set_explicit_caps (pad,
-      gst_caps_new_simple ("audio/x-raw-int",
-          "endianness", G_TYPE_INT, G_BYTE_ORDER,
-          "signed", G_TYPE_BOOLEAN, TRUE,
-          "width", G_TYPE_INT, 16,
-          "depth", G_TYPE_INT, 16,
-          "channels", G_TYPE_INT, channels, "rate", G_TYPE_INT, rate, NULL));
+  if (!channels)
+    return FALSE;
+
+  GST_INFO ("a52dec: reneg channels:%d rate:%d\n",
+      channels, a52dec->sample_rate);
+
+  caps = gst_caps_new_simple ("audio/x-raw-float",
+      "endianness", G_TYPE_INT, G_BYTE_ORDER,
+      "width", G_TYPE_INT, SAMPLE_WIDTH,
+      "channels", G_TYPE_INT, channels,
+      "rate", G_TYPE_INT, a52dec->sample_rate,
+      "buffer-frames", G_TYPE_INT, 0, NULL);
+  gst_audio_set_channel_positions (gst_caps_get_structure (caps, 0), pos);
+  g_free (pos);
+
+  return gst_pad_set_explicit_caps (pad, caps);
 }
 
 static void
-gst_a52dec_handle_event (GstA52Dec * a52dec)
+gst_a52dec_handle_event (GstA52Dec * a52dec, GstEvent * event)
 {
-  guint32 remaining;
-  GstEvent *event;
-
-  gst_bytestream_get_status (a52dec->bs, &remaining, &event);
-
-  if (!event) {
-    g_warning ("a52dec: no bytestream event");
-    return;
-  }
-
   GST_LOG ("Handling event of type %d timestamp %llu", GST_EVENT_TYPE (event),
       GST_EVENT_TIMESTAMP (event));
+
   switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_DISCONTINUOUS:
+    case GST_EVENT_DISCONTINUOUS:{
+      gint64 val;
+
+      if (!gst_event_discont_get_value (event, GST_FORMAT_TIME, &val)) {
+        GST_WARNING ("No time discont value in event %p", event);
+      } else {
+        a52dec->time = val;
+      }
+    }
+      /* fall-through */
     case GST_EVENT_FLUSH:
-      gst_bytestream_flush_fast (a52dec->bs, remaining);
+      if (a52dec->cache) {
+        gst_buffer_unref (a52dec->cache);
+        a52dec->cache = NULL;
+      }
       break;
     default:
       break;
   }
+
   gst_pad_event_default (a52dec->sinkpad, event);
 }
 
@@ -399,122 +354,135 @@ gst_a52dec_update_streaminfo (GstA52Dec * a52dec)
       GST_TAG_BITRATE, (guint) a52dec->bit_rate, NULL);
 
   gst_element_found_tags_for_pad (GST_ELEMENT (a52dec),
-      GST_PAD (a52dec->srcpad), a52dec->current_ts, taglist);
+      GST_PAD (a52dec->srcpad), a52dec->time, taglist);
 }
 
-static void
-gst_a52dec_loop (GstElement * element)
+static gboolean
+gst_a52dec_handle_frame (GstA52Dec * a52dec, guint8 * data,
+    guint length, gint flags, gint sample_rate, gint bit_rate)
 {
-  GstA52Dec *a52dec;
-  guint8 *data;
-  int i, length, flags, sample_rate, bit_rate;
-  int channels;
-  GstBuffer *buf;
-  guint32 got_bytes;
-  gboolean need_reneg;
-  GstClockTime timestamp = 0;
-
-  a52dec = GST_A52DEC (element);
-
-  /* find and read header */
-  do {
-    gint skipped_bytes = 0;
-
-    while (skipped_bytes < 3840) {
-      got_bytes = gst_bytestream_peek_bytes (a52dec->bs, &data, 7);
-      if (got_bytes < 7) {
-        gst_a52dec_handle_event (a52dec);
-        return;
-      }
-      length = a52_syncinfo (data, &flags, &sample_rate, &bit_rate);
-      if (length == 0) {
-        /* slide window to next 7 bytesa */
-        gst_bytestream_flush_fast (a52dec->bs, 1);
-        skipped_bytes++;
-        GST_LOG ("Skipped");
-      } else
-        break;
-    }
-  }
-  while (0);
+  gint channels, i;
+  gboolean need_reneg = FALSE;
 
+  /* update stream information, renegotiate or re-streaminfo if needed */
   need_reneg = FALSE;
-
   if (a52dec->sample_rate != sample_rate) {
     need_reneg = TRUE;
     a52dec->sample_rate = sample_rate;
   }
 
-  a52dec->stream_channels = flags & A52_CHANNEL_MASK;
+  if (flags) {
+    a52dec->stream_channels = flags & (A52_CHANNEL_MASK | A52_LFE);
+  }
 
   if (bit_rate != a52dec->bit_rate) {
     a52dec->bit_rate = bit_rate;
     gst_a52dec_update_streaminfo (a52dec);
   }
 
-  /* read the header + rest of frame */
-  got_bytes = gst_bytestream_read (a52dec->bs, &buf, length);
-  if (got_bytes < length) {
-    gst_a52dec_handle_event (a52dec);
-    return;
-  }
-  data = GST_BUFFER_DATA (buf);
-  timestamp = gst_bytestream_get_timestamp (a52dec->bs);
-  if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
-    if (timestamp == a52dec->last_ts) {
-      timestamp = a52dec->current_ts;
-    } else {
-      a52dec->last_ts = timestamp;
-    }
-  }
-
   /* process */
-  flags = a52dec->request_channels | A52_ADJUST_LEVEL;
+  flags = a52dec->request_channels;     /* | A52_ADJUST_LEVEL; */
   a52dec->level = 1;
-
   if (a52_frame (a52dec->state, data, &flags, &a52dec->level, a52dec->bias)) {
     GST_WARNING ("a52_frame error");
-    goto end;
+    return TRUE;
   }
-
-  channels = flags & A52_CHANNEL_MASK;
-
+  channels = flags & (A52_CHANNEL_MASK | A52_LFE);
   if (a52dec->using_channels != channels) {
     need_reneg = TRUE;
     a52dec->using_channels = channels;
   }
 
+  /* negotiate if required */
   if (need_reneg == TRUE) {
     GST_DEBUG ("a52dec reneg: sample_rate:%d stream_chans:%d using_chans:%d\n",
         a52dec->sample_rate, a52dec->stream_channels, a52dec->using_channels);
-    gst_a52dec_reneg (a52dec->srcpad,
-        gst_a52dec_channels (a52dec->using_channels), a52dec->sample_rate);
+    if (!gst_a52dec_reneg (a52dec->srcpad)) {
+      GST_ELEMENT_ERROR (a52dec, CORE, NEGOTIATION, (NULL), (NULL));
+      return FALSE;
+    }
   }
 
   if (a52dec->dynamic_range_compression == FALSE) {
     a52_dynrng (a52dec->state, NULL, NULL);
   }
 
+  /* each frame consists of 6 blocks */
   for (i = 0; i < 6; i++) {
     if (a52_block (a52dec->state)) {
       GST_WARNING ("a52_block error %d", i);
-      continue;
-    }
-    /* push on */
-
-    if (gst_a52dec_push (a52dec->srcpad, a52dec->using_channels,
-            a52dec->samples, timestamp)) {
-      GST_WARNING ("a52dec push error");
     } else {
+      /* push on */
+      gst_a52dec_push (a52dec->srcpad, a52dec->using_channels,
+          a52dec->samples, a52dec->time);
+    }
+    a52dec->time += 256 * GST_SECOND / a52dec->sample_rate;
+  }
 
-      if (i % 2)
-        timestamp += 256 * GST_SECOND / a52dec->sample_rate;
+  return TRUE;
+}
+
+static void
+gst_a52dec_chain (GstPad * pad, GstData * _data)
+{
+  GstA52Dec *a52dec = GST_A52DEC (gst_pad_get_parent (pad));
+  GstBuffer *buf;
+  guint8 *data;
+  guint size;
+  gint length = 0, flags, sample_rate, bit_rate;
+
+  /* event handling */
+  if (GST_IS_EVENT (_data)) {
+    gst_a52dec_handle_event (a52dec, GST_EVENT (_data));
+    return;
+  }
+
+  /* merge with cache, if any. Also make sure timestamps match */
+  buf = GST_BUFFER (_data);
+  if (GST_BUFFER_TIMESTAMP_IS_VALID (buf)) {
+    a52dec->time = GST_BUFFER_TIMESTAMP (buf);
+  }
+  if (a52dec->cache) {
+    buf = gst_buffer_join (a52dec->cache, buf);
+    a52dec->cache = NULL;
+  }
+  data = GST_BUFFER_DATA (buf);
+  size = GST_BUFFER_SIZE (buf);
+
+  /* find and read header */
+  bit_rate = a52dec->bit_rate;
+  sample_rate = a52dec->sample_rate;
+  flags = 0;
+  while (size >= 7) {
+    length = a52_syncinfo (data, &flags, &sample_rate, &bit_rate);
+    if (length == 0) {
+      /* no sync */
+      data++;
+      size--;
+    } else if (length <= size) {
+      GST_DEBUG ("Sync: %d", length);
+      if (!gst_a52dec_handle_frame (a52dec, data,
+              length, flags, sample_rate, bit_rate)) {
+        size = 0;
+        break;
+      }
+      size -= length;
+      data += length;
+    } else {
+      /* not enough data */
+      GST_LOG ("Not enough data available");
+      break;
     }
   }
 
-  a52dec->current_ts = timestamp;
-
-end:
+  /* keep cache */
+  if (length == 0) {
+    GST_LOG ("No sync found");
+  }
+  if (size > 0) {
+    a52dec->cache = gst_buffer_create_sub (buf,
+        GST_BUFFER_SIZE (buf) - size, size);
+  }
   gst_buffer_unref (buf);
 }
 
@@ -527,7 +495,6 @@ gst_a52dec_change_state (GstElement * element)
 
   switch (GST_STATE_TRANSITION (element)) {
     case GST_STATE_NULL_TO_READY:
-      a52dec->bs = gst_bytestream_new (a52dec->sinkpad);
       cpuflags = gst_cpu_get_flags ();
       if (cpuflags & GST_CPU_FLAG_MMX)
         a52_cpuflags |= MM_ACCEL_X86_MMX;
@@ -543,25 +510,22 @@ gst_a52dec_change_state (GstElement * element)
       a52dec->bit_rate = -1;
       a52dec->sample_rate = -1;
       a52dec->stream_channels = A52_CHANNEL;
-      /* FIXME force stereo for now */
-      a52dec->request_channels = A52_STEREO;
+      a52dec->request_channels = A52_3F2R | A52_LFE;
       a52dec->using_channels = A52_CHANNEL;
       a52dec->level = 1;
-      a52dec->bias = 384;
-      a52dec->last_ts = 0;
-      a52dec->current_ts = 0;
-      a52dec->last_timestamp = 0;
-      a52dec->last_diff = 0;
-
+      a52dec->bias = 0;
+      a52dec->time = 0;
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
       break;
     case GST_STATE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_PAUSED_TO_READY:
-      gst_bytestream_destroy (a52dec->bs);
-      a52dec->bs = NULL;
       a52dec->samples = NULL;
+      if (a52dec->cache) {
+        gst_buffer_unref (a52dec->cache);
+        a52dec->cache = NULL;
+      }
       break;
     case GST_STATE_READY_TO_NULL:
       a52_free (a52dec->state);
@@ -621,7 +585,7 @@ static gboolean
 plugin_init (GstPlugin * plugin)
 {
 
-  if (!gst_library_load ("gstbytestream"))
+  if (!gst_library_load ("gstaudio"))
     return FALSE;
 
   if (!gst_element_register (plugin, "a52dec", GST_RANK_PRIMARY,
