@@ -31,7 +31,7 @@ static GstPadTemplate *dec_src_template, *dec_sink_template;
 /* elementfactory information */
 GstElementDetails tarkindec_details = {
   "Ogg Tarkin decoder",
-  "Filter/Video/Decoder",
+  "Codec/Decoder/Video",
   "Decodes video in OGG Tarkin format",
   "Monty <monty@xiph.org>, " "Wim Taymans <wim.taymans@chello.be>",
 };
@@ -46,7 +46,7 @@ enum
 enum
 {
   ARG_0,
-  ARG_BITRATE,
+  ARG_BITRATE
 };
 
 static void gst_tarkindec_base_init (gpointer g_class);
