@@ -133,6 +133,22 @@ gst_resample_reinit (gst_resample_t * r)
   }
 }
 
+void
+gst_resample_close (gst_resample_t * r)
+{
+  if (r->buffer) {
+    free (r->buffer);
+    r->buffer = NULL;
+    r->buffer_len = 0;
+  }
+  if (r->hack_union.s.out_tmp) {
+    free (r->hack_union.s.out_tmp);
+    r->hack_union.s.out_tmp = NULL;
+    r->hack_union.s.out_tmp_len = 0;
+  }
+
+}
+
 /*
  * Prepare to be confused.
  *
@@ -252,8 +268,7 @@ gst_resample_nearest_s16 (gst_resample_t * r)
       SCALE_LOOP (o_ptr[0] = i_ptr[0], 1);
       break;
     case 2:
-      SCALE_LOOP (o_ptr[0] = i_ptr[0];
-          o_ptr[1] = i_ptr[1], 2);
+      SCALE_LOOP (o_ptr[0] = i_ptr[0]; o_ptr[1] = i_ptr[1], 2);
       break;
     default:
     {
@@ -612,8 +627,7 @@ gst_resample_nearest_float (gst_resample_t * r)
       SCALE_LOOP (o_ptr[0] = i_ptr[0], 1);
       break;
     case 2:
-      SCALE_LOOP (o_ptr[0] = i_ptr[0];
-          o_ptr[1] = i_ptr[1], 2);
+      SCALE_LOOP (o_ptr[0] = i_ptr[0]; o_ptr[1] = i_ptr[1], 2);
       break;
     default:
     {
