@@ -19,7 +19,7 @@
 /* Element-Checklist-Version: 5 */
 
 /* 2001/04/03 - Updated parseau to use caps nego
- *              Zaheer Merali <zaheer@grid9.net
+ *              Zaheer Abbas Merali <zaheerabbas at merali dot org>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,10 +49,19 @@ static GstStaticPadTemplate gst_auparse_src_template =
     GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_SOMETIMES,          /* FIXME: spider */
-    GST_STATIC_CAPS (GST_AUDIO_INT_PAD_TEMPLATE_CAPS "; "       /* 24-bit PCM is barely supported by gstreamer actually */
-        GST_AUDIO_FLOAT_PAD_TEMPLATE_CAPS "; "  /* 64-bit float is barely supported by gstreamer actually */
-        "audio/x-alaw, " "rate = (int) [ 8000, 192000 ], "
-        "channels = (int) [ 1, 2 ]" "; " "audio/x-mulaw, "
+    GST_STATIC_CAPS (GST_AUDIO_INT_PAD_TEMPLATE_CAPS "; "
+        /* we don't use GST_AUDIO_FLOAT_PAD_TEMPLATE_CAPS
+           because of min buffer-frames which is 1, not 0 */
+        "audio/x-raw-float, "
+        "rate = (int) [ 1, MAX ], "
+        "channels = (int) [ 1, MAX ], "
+        "endianness = (int) { LITTLE_ENDIAN , BIG_ENDIAN }, "
+        "width = (int) { 32, 64 }, "
+        "buffer-frames = (int) [ 0, MAX]" "; "
+        "audio/x-alaw, "
+        "rate = (int) [ 8000, 192000 ], "
+        "channels = (int) [ 1, 2 ]" "; "
+        "audio/x-mulaw, "
         "rate = (int) [ 8000, 192000 ], " "channels = (int) [ 1, 2 ]" "; "
         /* Nothing to decode those ADPCM streams for now */
         "audio/x-adpcm, " "layout = (string) { g721, g722, g723_3, g723_5 }")
@@ -221,7 +230,7 @@ gst_auparse_chain (GstPad * pad, GstData * _data)
       head++;
 
     } else {
-      g_warning ("help, dunno what I'm looking at!\n");
+      GST_ELEMENT_ERROR (auparse, STREAM, WRONG_TYPE, (NULL), (NULL));
       gst_buffer_unref (buf);
       return;
     }
@@ -320,26 +329,30 @@ Samples :
     if (law) {
       tempcaps =
           gst_caps_new_simple ((law == 1) ? "audio/x-mulaw" : "audio/x-alaw",
-          "rate", G_TYPE_INT, auparse->frequency, "channels", G_TYPE_INT,
-          auparse->channels, NULL);
+          "rate", G_TYPE_INT, auparse->frequency,
+          "channels", G_TYPE_INT, auparse->channels, NULL);
     } else if (ieee) {
       tempcaps = gst_caps_new_simple ("audio/x-raw-float",
-          "width", G_TYPE_INT, depth,
+          "rate", G_TYPE_INT, auparse->frequency,
+          "channels", G_TYPE_INT, auparse->channels,
           "endianness", G_TYPE_INT,
-          auparse->le ? G_LITTLE_ENDIAN : G_BIG_ENDIAN, NULL);
+          auparse->le ? G_LITTLE_ENDIAN : G_BIG_ENDIAN, "width", G_TYPE_INT,
+          depth, "buffer-frames", G_TYPE_INT, 0, NULL);
     } else if (layout[0]) {
       tempcaps = gst_caps_new_simple ("audio/x-adpcm",
           "layout", G_TYPE_STRING, layout, NULL);
     } else {
       tempcaps = gst_caps_new_simple ("audio/x-raw-int",
+          "rate", G_TYPE_INT, auparse->frequency,
+          "channels", G_TYPE_INT, auparse->channels,
           "endianness", G_TYPE_INT,
-          auparse->le ? G_LITTLE_ENDIAN : G_BIG_ENDIAN, "rate", G_TYPE_INT,
-          auparse->frequency, "channels", G_TYPE_INT, auparse->channels,
-          "depth", G_TYPE_INT, depth, "width", G_TYPE_INT, depth, "signed",
-          G_TYPE_BOOLEAN, TRUE, NULL);
+          auparse->le ? G_LITTLE_ENDIAN : G_BIG_ENDIAN, "depth", G_TYPE_INT,
+          depth, "width", G_TYPE_INT, depth, "signed", G_TYPE_BOOLEAN, TRUE,
+          NULL);
     }
 
     if (!gst_pad_set_explicit_caps (auparse->srcpad, tempcaps)) {
+      GST_ELEMENT_ERROR (auparse, CORE, NEGOTIATION, (NULL), (NULL));
       gst_buffer_unref (buf);
       gst_object_unref (GST_OBJECT (auparse->srcpad));
       auparse->srcpad = NULL;
@@ -399,4 +412,4 @@ plugin_init (GstPlugin * plugin)
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "auparse",
-    "parses au streams", plugin_init, VERSION, "GPL", GST_PACKAGE, GST_ORIGIN)
+    "parses au streams", plugin_init, VERSION, "LGPL", GST_PACKAGE, GST_ORIGIN)
