@@ -1191,6 +1191,7 @@ gst_avi_demux_stream_index (GstAviDemux * avi,
   guint32 tag;
   guint index_size;
   gst_avi_index_entry *index_entries = NULL;
+  gboolean first = TRUE;
 
   /* first, we need to know the current position (to seek back
    * when we're done) and the total length of the file. */
@@ -1198,17 +1199,29 @@ gst_avi_demux_stream_index (GstAviDemux * avi,
   pos_before = gst_bytestream_tell (riff->bs);
 
   /* skip movi
-   *
-   * FIXME:
-   * - we want to add error handling here so we can recover.
    */
+  if (pos_before + 8 > length) {
+    return TRUE;
+  } else {
+    guint8 *data;
+
+    if (gst_bytestream_peek_bytes (riff->bs, &data, 8) == 8) {
+      guint len = GST_READ_UINT32_LE (&data[4]);
+
+      if (pos_before + 8 + len >= length) {
+        GST_WARNING ("No index avail");
+        return TRUE;
+      }
+    }
+  }
+  /* hmm... */
   if (!gst_riff_read_skip (riff))
     return FALSE;
 
   /* assure that we've got data left */
   pos_after = gst_bytestream_tell (riff->bs);
   if (pos_after + 8 > length) {
-    g_warning ("File said that it has an index, but there is no index data!");
+    GST_WARNING ("File said that it has an index, but there is no index data!");
     goto end;
   }
 
@@ -1244,7 +1257,10 @@ gst_avi_demux_stream_index (GstAviDemux * avi,
     entry.size = GUINT32_FROM_LE (_entry->size);
     target = &index_entries[i];
 
-    if (entry.id == GST_RIFF_rec || entry.id == 0 || entry.size == 0)
+    /* Don't index reclists. ID = 0 and offset = 0 (where this is not the
+     * first chunk) indicate broken indexes. Rebuild from there. */
+    if (entry.id == GST_RIFF_rec || entry.id == 0 ||
+        (entry.offset == 0 && i > 0))
       continue;
 
     stream_nr = CHUNKID_TO_STREAMNR (entry.id);
@@ -1262,11 +1278,12 @@ gst_avi_demux_stream_index (GstAviDemux * avi,
     target->offset = entry.offset + 8;
 
     /* figure out if the index is 0 based or relative to the MOVI start */
-    if (i == 0) {
+    if (first) {
       if (target->offset < pos_before)
         avi->index_offset = pos_before + 8;
       else
         avi->index_offset = 0;
+      first = FALSE;
     }
 
     target->bytes_before = stream->total_bytes;
@@ -1296,8 +1313,11 @@ gst_avi_demux_stream_index (GstAviDemux * avi,
     stream->total_bytes += target->size;
     stream->total_frames++;
 
-    GST_DEBUG ("Adding index entry of size %u at offset %"
-        G_GUINT64_FORMAT, target->size, target->offset);
+    GST_DEBUG ("Adding index entry %d (%d) for stream %d of size %u "
+        "at offset %" G_GUINT64_FORMAT " and time %" GST_TIME_FORMAT,
+        target->index_nr, stream->total_frames - 1,
+        target->stream_nr, target->size, target->offset,
+        GST_TIME_ARGS (target->ts));
     list = g_list_prepend (list, target);
   }
 
@@ -1569,6 +1589,10 @@ gst_avi_demux_stream_scan (GstAviDemux * avi,
     stream->total_frames++;
 
     list = g_list_prepend (list, entry);
+    GST_DEBUG ("Added index entry %d (in stream: %d), offset %"
+        G_GUINT64_FORMAT ", time %" GST_TIME_FORMAT " for stream %d",
+        index_size - 1, entry->frames_before, entry->offset,
+        GST_TIME_ARGS (entry->ts), entry->stream_nr);
 
   next:
     if (!gst_avi_demux_skip (avi, TRUE))
@@ -1623,16 +1647,24 @@ gst_avi_demux_massage_index (GstAviDemux * avi,
 
   /* init frames */
   for (i = 0; i < avi->num_streams; i++) {
+    GstFormat fmt = GST_FORMAT_TIME;
+
     stream = &avi->stream[i];
     if (stream->strh->type == GST_RIFF_FCC_vids) {
-      if (stream->strh->rate != 0)
-        stream->delay = stream->strh->init_frames * GST_SECOND *
-            stream->strh->scale / stream->strh->rate;
+      if (!gst_pad_convert (stream->pad,
+              GST_FORMAT_DEFAULT, stream->strh->init_frames,
+              &fmt, &stream->delay)) {
+        stream->delay = 0;
+      }
     } else {
-      if (stream->total_frames * stream->bitrate != 0)
-        stream->delay = GST_SECOND * stream->strh->init_frames *
-            stream->strh->length / (stream->total_frames * stream->bitrate);
+      if (!gst_pad_convert (stream->pad,
+              GST_FORMAT_DEFAULT, stream->strh->init_frames,
+              &fmt, &stream->delay)) {
+        stream->delay = 0;
+      }
     }
+    GST_LOG ("Adding init_time=%" GST_TIME_FORMAT " to stream %d",
+        GST_TIME_ARGS (stream->delay), i);
   }
   for (one = list; one != NULL; one = one->next) {
     entry = one->data;
@@ -1922,6 +1954,7 @@ gst_avi_demux_handle_seek (GstAviDemux * avi)
   /* FIXME: if we seek in an openDML file, we will have multiple
    * primary levels. Seeking in between those will cause havoc. */
 
+  GST_LOG ("Seeking to entry %d", avi->seek_entry);
   avi->current_entry = avi->seek_entry;
 
   for (i = 0; i < avi->num_streams; i++) {
@@ -1945,13 +1978,14 @@ static gboolean
 gst_avi_demux_process_next_entry (GstAviDemux * avi)
 {
   GstRiffRead *riff = GST_RIFF_READ (avi);
-  gboolean processed;
+  gboolean processed = FALSE;
 
-  for (processed = FALSE; !processed;) {
+  do {
     if (avi->current_entry >= avi->index_size) {
       gst_bytestream_seek (riff->bs, 0, GST_SEEK_METHOD_END);
 
       /* get eos */
+      GST_LOG ("Handled last index entry, setting EOS");
       gst_riff_peek_tag (GST_RIFF_READ (avi), &avi->level_up);
       gst_pad_event_default (avi->sinkpad, gst_event_new (GST_EVENT_EOS));
       processed = TRUE;
@@ -1962,6 +1996,7 @@ gst_avi_demux_process_next_entry (GstAviDemux * avi)
       avi_stream_context *stream;
 
       if (entry->stream_nr >= avi->num_streams) {
+        GST_DEBUG ("Entry has non-existing stream nr %d", entry->stream_nr);
         continue;
       }
 
@@ -1987,6 +2022,7 @@ gst_avi_demux_process_next_entry (GstAviDemux * avi)
           }
         }
         if (!(buf = gst_riff_read_element_data (riff, entry->size, &got))) {
+          GST_ERROR ("Failed to read %d bytes of data", entry->size);
           return FALSE;
         }
         if (entry->flags & GST_RIFF_IF_KEYFRAME) {
@@ -2000,11 +2036,14 @@ gst_avi_demux_process_next_entry (GstAviDemux * avi)
             gst_pad_get_name (stream->pad));
         gst_pad_push (stream->pad, GST_DATA (buf));
         processed = TRUE;
+      } else {
+        GST_DEBUG ("Unusable pad or zero chunksize, skipping entry");
+        processed = TRUE;
       }
-      stream->current_frame++;
-      stream->current_byte += entry->size;
+      stream->current_frame = entry->frames_before + 1;
+      stream->current_byte = entry->bytes_before + entry->size;
     }
-  }
+  } while (!processed);
 
   return TRUE;
 }
