@@ -230,6 +230,7 @@ gst_ffmpegcsp_pad_link (GstPad * pad, const GstCaps * caps)
           gst_value_get_fraction_denominator (par), NULL);
     }
     ret = gst_pad_try_set_caps (otherpad, caps);
+    gst_caps_free (caps);
     if (GST_PAD_LINK_FAILED (ret)) {
       return ret;
     }
@@ -357,9 +358,8 @@ gst_ffmpegcsp_chain (GstPad * pad, GstData * data)
   if (space->from_pixfmt == space->to_pixfmt) {
     outbuf = inbuf;
   } else {
-#define ROUND_UP_4(x) (((x) + 3) & ~3)
     guint size = avpicture_get_size (space->to_pixfmt,
-        ROUND_UP_4 (space->width), ROUND_UP_4 (space->height));
+        space->width, space->height);
 
     outbuf = gst_pad_alloc_buffer (space->srcpad, GST_BUFFER_OFFSET_NONE, size);
 
