@@ -159,6 +159,7 @@ static void gst_dvd_demux_set_cur_audio
 static void gst_dvd_demux_set_cur_subpicture
     (GstDVDDemux * dvd_demux, gint stream_nr);
 
+static GstElementStateReturn gst_dvd_demux_change_state (GstElement * element);
 
 static GstMPEGDemuxClass *parent_class = NULL;
 
@@ -243,6 +244,8 @@ gst_dvd_demux_class_init (GstDVDDemuxClass * klass)
   mpeg_parse_class = (GstMPEGParseClass *) klass;
   mpeg_demux_class = (GstMPEGDemuxClass *) klass;
 
+  gstelement_class->change_state = gst_dvd_demux_change_state;
+
   mpeg_parse_class->send_discont = gst_dvd_demux_send_discont;
 
   mpeg_demux_class->get_audio_stream = gst_dvd_demux_get_audio_stream;
@@ -596,6 +599,9 @@ gst_dvd_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
 
 
         lpcm_str->sample_info = sample_info;
+        lpcm_str->width = width;
+        lpcm_str->rate = rate;
+        lpcm_str->channels = channels;
         break;
 
       case GST_DVD_DEMUX_AUDIO_AC3:
@@ -690,6 +696,7 @@ gst_dvd_demux_process_private (GstMPEGDemux * mpeg_demux,
   guint8 ps_id_code, lpcm_sample_info;
   GstMPEGStream *outstream = NULL;
   guint first_access = 0;
+  gint align = 1, len, off;
 
   basebuf = GST_BUFFER_DATA (buffer);
 
@@ -718,11 +725,14 @@ gst_dvd_demux_process_private (GstMPEGDemux * mpeg_demux,
         headerlen += 4;
         datalen -= 4;
       } else if (ps_id_code >= 0xA0 && ps_id_code <= 0xA7) {
+        GstDVDLPCMStream *lpcm_str;
+
         GST_LOG_OBJECT (dvd_demux,
             "we have an audio (LPCM) packet, track %d", ps_id_code - 0xA0);
         lpcm_sample_info = basebuf[headerlen + 9];
         outstream = DEMUX_CLASS (dvd_demux)->get_audio_stream (mpeg_demux,
             ps_id_code - 0xA0, GST_DVD_DEMUX_AUDIO_LPCM, &lpcm_sample_info);
+        lpcm_str = (GstDVDLPCMStream *) outstream;
 
         /* Determine the position of the "first access". */
         first_access = *(basebuf + headerlen + 6) * 256 +
@@ -731,6 +741,9 @@ gst_dvd_demux_process_private (GstMPEGDemux * mpeg_demux,
         /* Get rid of the LPCM header. */
         headerlen += 7;
         datalen -= 7;
+
+        /* align by samples */
+        align = lpcm_str->width * lpcm_str->channels / 8;
       } else if (ps_id_code >= 0x20 && ps_id_code <= 0x3F) {
         GST_LOG_OBJECT (dvd_demux,
             "we have a subpicture packet, track %d", ps_id_code - 0x20);
@@ -784,14 +797,29 @@ gst_dvd_demux_process_private (GstMPEGDemux * mpeg_demux,
        a means to associate a timestamp to the middle of a buffer, we
        send two separate buffers and put the timestamp in the second
        one. */
-    DEMUX_CLASS (dvd_demux)->send_subbuffer (mpeg_demux, outstream,
-        buffer, GST_CLOCK_TIME_NONE, headerlen + 4, first_access - 1);
-    DEMUX_CLASS (dvd_demux)->send_subbuffer (mpeg_demux, outstream,
-        buffer, timestamp,
-        headerlen + 3 + first_access, datalen - (first_access - 1));
+    off = headerlen + 4;
+    len = first_access - 1;
+    len -= len % align;
+    if (len > 0) {
+      DEMUX_CLASS (dvd_demux)->send_subbuffer (mpeg_demux, outstream,
+          buffer, GST_CLOCK_TIME_NONE, headerlen + 4, first_access - 1);
+    }
+    off += len;
+    len = datalen - len;
+    len -= len % align;
+    if (len > 0) {
+      DEMUX_CLASS (dvd_demux)->send_subbuffer (mpeg_demux, outstream,
+          buffer, timestamp,
+          headerlen + 3 + first_access, datalen - (first_access - 1));
+    }
   } else {
-    DEMUX_CLASS (dvd_demux)->send_subbuffer (mpeg_demux, outstream,
-        buffer, timestamp, headerlen + 4, datalen);
+    off = headerlen + 4;
+    len = datalen;
+    len -= len % align;
+    if (len > 0) {
+      DEMUX_CLASS (dvd_demux)->send_subbuffer (mpeg_demux, outstream,
+          buffer, timestamp, headerlen + 4, datalen);
+    }
   }
 }
 
@@ -921,6 +949,50 @@ gst_dvd_demux_set_cur_subpicture (GstDVDDemux * dvd_demux, gint stream_nr)
   }
 }
 
+static void
+gst_dvd_demux_reset (GstDVDDemux * dvd_demux)
+{
+  int i;
+
+  GST_INFO ("Resetting the dvd demuxer");
+  for (i = 0; i < GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS; i++) {
+    if (dvd_demux->subpicture_stream[i]) {
+      gst_element_remove_pad (GST_ELEMENT (dvd_demux),
+          dvd_demux->subpicture_stream[i]->pad);
+      g_free (dvd_demux->subpicture_stream[i]);
+      dvd_demux->subpicture_stream[i] = NULL;
+    }
+    dvd_demux->subpicture_time[i] = 0;
+  }
+  gst_pad_set_explicit_caps (dvd_demux->cur_video, NULL);
+  gst_pad_set_explicit_caps (dvd_demux->cur_audio, NULL);
+  gst_pad_set_explicit_caps (dvd_demux->cur_subpicture, NULL);
+
+  dvd_demux->cur_video_nr = 0;
+  dvd_demux->cur_audio_nr = 0;
+  dvd_demux->cur_subpicture_nr = 0;
+  dvd_demux->mpeg_version = 0;
+  dvd_demux->last_end_ptm = 0;
+
+  dvd_demux->just_flushed = FALSE;
+  dvd_demux->discont_time = GST_CLOCK_TIME_NONE;
+}
+
+static GstElementStateReturn
+gst_dvd_demux_change_state (GstElement * element)
+{
+  GstDVDDemux *dvd_demux = GST_DVD_DEMUX (element);
+
+  switch (GST_STATE_TRANSITION (element)) {
+    case GST_STATE_PAUSED_TO_READY:
+      gst_dvd_demux_reset (dvd_demux);
+      break;
+    default:
+      break;
+  }
+
+  return GST_ELEMENT_CLASS (parent_class)->change_state (element);
+}
 
 gboolean
 gst_dvd_demux_plugin_init (GstPlugin * plugin)
