@@ -190,6 +190,31 @@ id3_type_find (GstTypeFind * tf, gpointer unused)
   }
 }
 
+/*** application/x-ape **************************************************************/
+
+static GstStaticCaps apetag_caps = GST_STATIC_CAPS ("application/x-apetag");
+
+#define APETAG_CAPS gst_static_caps_get(&apetag_caps)
+static void
+apetag_type_find (GstTypeFind * tf, gpointer unused)
+{
+  guint8 *data;
+
+  /* APEv1/2 at start of file */
+  data = gst_type_find_peek (tf, 0, 8);
+  if (data && !memcmp (data, "APETAGEX", 8)) {
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, APETAG_CAPS);
+    return;
+  }
+
+  /* APEv1/2 at end of file */
+  data = gst_type_find_peek (tf, -32, 8);
+  if (data && !memcmp (data, "APETAGEX", 8)) {
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, APETAG_CAPS);
+    return;
+  }
+}
+
 /*** audio/x-ttafile *********************************************************/
 
 static GstStaticCaps tta_caps = GST_STATIC_CAPS ("audio/x-ttafile");
@@ -369,14 +394,14 @@ static GstStaticCaps mp3_caps = GST_STATIC_CAPS ("audio/mpeg, "
 static void
 mp3_type_find (GstTypeFind * tf, gpointer unused)
 {
-  /*guint64 length = gst_type_find_get_length (tf); */
+  guint64 length = gst_type_find_get_length (tf);
   gint try;
   guint8 *data = NULL;
   guint size;
   guint64 skipped;
 
-  for (try = 0; try < 1 /* 2 */ ; try++) {
-    guint64 start_off = 0;      /*(try == 0) ? 0 : length / 2; */
+  for (try = 0; try < 2; try++) {
+    guint64 start_off = (try == 0) ? 0 : length / 2;
 
     if (try != 0 && start_off == 0)
       return;
@@ -519,6 +544,13 @@ static GstStaticCaps mpeg_sys_caps = GST_STATIC_CAPS ("video/mpeg, "
 					 (((guint8 *)data)[1] == 0x00) &&	\
 					 (((guint8 *)data)[2] == 0x01) &&	\
 					 ((((guint8 *)data)[3] & 0x80) == 0x80))
+#define IS_MPEG_PES_HEADER(data)        ((((guint8 *)data)[0] == 0x00) &&	\
+					 (((guint8 *)data)[1] == 0x00) &&	\
+					 (((guint8 *)data)[2] == 0x01) && \
+					 ((((guint8 *)data)[3] == 0xE0) || \
+					  (((guint8 *)data)[3] == 0xC0) || \
+					  (((guint8 *)data)[3] == 0xBD)))
+
 static void
 mpeg2_sys_type_find (GstTypeFind * tf, gpointer unused)
 {
@@ -540,7 +572,15 @@ mpeg2_sys_type_find (GstTypeFind * tf, gpointer unused)
       gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, caps);
       gst_caps_free (caps);
     }
+  } else if (data && IS_MPEG_PES_HEADER (data)) {
+    /* PES stream */
+    GstCaps *caps = gst_caps_copy (MPEG_SYS_CAPS);
+
+    gst_structure_set (gst_caps_get_structure (caps, 0), "mpegversion",
+        G_TYPE_INT, 2, 0);
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, caps);
   }
+
 };
 
 /* ATTANTION: ugly return value:
@@ -1406,6 +1446,7 @@ plugin_init (GstPlugin * plugin)
   static gchar *flx_exts[] = { "flc", "fli", NULL };
   static gchar *id3_exts[] =
       { "mp3", "mp2", "mp1", "mpga", "ogg", "flac", "tta", NULL };
+  static gchar *apetag_exts[] = { "ape", "mpc", NULL };
   static gchar *tta_exts[] = { "tta", NULL };
   static gchar *mod_exts[] = { "669", "amf", "dsm", "gdm", "far", "imf",
     "it", "med", "mod", "mtm", "okt", "sam",
@@ -1463,12 +1504,17 @@ plugin_init (GstPlugin * plugin)
       avi_exts, "AVI ");
   TYPE_FIND_REGISTER_RIFF (plugin, "video/x-cdxa", GST_RANK_PRIMARY,
       cdxa_exts, "CDXA");
+  TYPE_FIND_REGISTER_START_WITH (plugin, "video/x-vcd", GST_RANK_PRIMARY,
+      cdxa_exts, "\000\377\377\377\377\377\377\377\377\377\377\000", 12,
+      GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-flac", GST_RANK_PRIMARY,
       flac_exts, "fLaC", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "video/x-fli", GST_RANK_MARGINAL, flx_type_find,
       flx_exts, FLX_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "application/x-id3", GST_RANK_PRIMARY,
       id3_type_find, id3_exts, ID3_CAPS, NULL);
+  TYPE_FIND_REGISTER (plugin, "application/x-apetag", GST_RANK_PRIMARY,
+      apetag_type_find, apetag_exts, APETAG_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/x-ttafile", GST_RANK_PRIMARY,
       tta_type_find, tta_exts, TTA_CAPS, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/x-mod", GST_RANK_SECONDARY, mod_type_find,
