@@ -34,19 +34,11 @@
 #include "gstv4lmjpegsink.h"
 
 GST_DEBUG_CATEGORY (v4l_debug); /* used in v4l_calls.c and v4lsrc_calls.c */
-GST_DEBUG_CATEGORY_EXTERN (v4loverlay_debug);
 
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
   GST_DEBUG_CATEGORY_INIT (v4l_debug, "v4l", 0, "V4L API calls");
-  GST_DEBUG_CATEGORY_INIT (v4loverlay_debug, "v4loverlay", 0,
-      "V4L overlay calls");
-
-  /* actually, we can survive without it, but I'll create
-   * that handling later on. */
-  if (!gst_library_load ("xwindowlistener"))
-    return FALSE;
 
   if (!gst_element_register (plugin, "v4lelement",
           GST_RANK_NONE, GST_TYPE_V4LELEMENT) ||
